package org.enfi.modules.quartz.job;

import lombok.extern.slf4j.Slf4j;
import org.enfi.common.util.DateUtils;
import org.enfi.modules.system.service.IDepartDataService;
import org.enfi.modules.system.service.IOaEmployeeVService;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @Description: 同步定时任务测试
 * <p>
 * 此处的同步是指 当定时任务的执行时间大于任务的时间间隔时
 * 会等待第一个任务执行完成才会走第二个任务
 * @author: taoyan
 * @date: 2020年06月19日
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Slf4j
public class AsyncErpJob implements Job {

    @Autowired
    private IOaEmployeeVService iOaEmployeeVService;
    @Autowired
    private IDepartDataService iDepartDataService;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        log.info(" --- 同步任务调度开始 --- ");
        iDepartDataService.importDep();
        iOaEmployeeVService.importSysUserData();
        //测试发现 每5秒执行一次
        log.info(" --- 执行完毕，时间：" + DateUtils.now() + "---");
    }

}
